/*###################################################################
### Khlschranktr-Alarm 03/2007 (c) Uwe Freese                   ###
### http://www.uwe-freese.de                   mail@uwe-freese.de ###
###                                                               ###
### You can use this program freely, but please always include    ###
### this copyright information when you share it.                 ###
###                                                               ###
### Diese Software kann frei verwendet werden, aber bitte immer   ###
### diesen Copyright-Hinweis hinzufgen, wenn es weitergegeben    ###
### wird!                                                         ###
###################################################################*/

#include <delay.h>
#include <sleep.h>

#include "tueralarm.h"

static int i;

static void io_init(void)
{
	// PortB
	PORTB = 0b00000000;
	DDRB  = 0b00000000; 
	// PortC
	// DDRC = Data Direction Register
	// Pin 0 = Ausgang Summer
	// Pin 1 = Ausgang LED
	// Pin 2 = Eingang Schalter
	PORTC = 0b00000000;
	DDRC  = 0b00000011;
	// PortD
	PORTD = 0b00000000;
	DDRD  = 0b00000000;

	// Watchdog Disabled (also to reduce power consumption)
	wdt_disable();
	
	// INT0 Enabled, Mode: Low Level
	MCUCR |= 0x00;
	GICR = 0x40;
	   	  
	// configure power down mode
	cbi(MCUCR,4);
	sbi(MCUCR,5);
	cbi(MCUCR,6);
	
	// disable interrupt as default
	cli();

	// Analog Comparator Disabled
	ACSR = 0x80;
}

void timers_init(void)
{
	// Timer/Counter0 Clock source: System Clock
	// Timer/Counter0 Clock value: Stopped
	TCNT0 = 0x00;
	TCCR0 = 0x00;

	// Timer/Counter1 Clock source: System Clock
	// Timer/Counter1 Clock value: Stopped
	// Timer/Counter1 Mode: Normal
	// Timer/Counter1 Output: A: Disconnected, B: Disconnected
	OCR1A = 0x0000;
	OCR1B = 0x0000;
	TCNT1 = 0x0000;
	TCCR1A = 0x00;
	TCCR1B = 0x00;

	// Timer/Counter2 Clock source: TOSC1
	// Timer/Counter2 Clock value: PCK2/128
	// Timer/Counter2 Mode: CTC, TOP=OCR2
	// Timer/Counter2 Output: Disconnected
	ASSR = 0x08;
	OCR2 = 0x80;
	TCNT2 = 0x80;
	TCCR2 = 0x0e;

	TIMSK = 0x80;
}

inline void led_blink(short count, short pause)
{
	for (i = 0; i < count; i++)
	{
		LED_ON
		sleep(pause);
		LED_OFF
		sleep(pause);
	}
}

inline void buzzer_blink(short count, short pause)
{
	for (i = 0; i < count; i++)
	{
		BUZZER_ON
		sleep(pause);
		BUZZER_OFF
		sleep(pause);
	}
}

SIGNAL(SIG_INTERRUPT0)
{
	// nothing to do here, interrupt is only used to wake up
}

// go to sleep and wait until the user wakes up by pressing the button
void go_to_sleep(void)
{
  // clear display
  BUZZER_OFF
  LED_OFF
  
  sei(); // enable interrupts to make wakeup possible
  sbi(MCUCR, SE); // Sleep enable bit makes "sleep" asm instruction available
  asm volatile("sleep"); // go to sleep
  
  // after wakeup
  
  cbi(MCUCR, SE); // disable sleep instruction
  cli(); // disable interrupts
}

// check if switch is active long enough
bool switch_active_long(void)
{
	int sec = 0;
	
	while (SWITCH)
	{
		led_blink(1,10);
		sleep(990);
		sec++;
		
		if (ALARM_DELAY_SEC == sec)
		{
			return true;
		}
	}
	
	return false;
}

void alarm(void)
{
	unsigned long alarm_cycle = 1;
	unsigned long tick;
			
	while (SWITCH)
	{
		// buzz
					
		buzzer_blink(1, 1000);

		// calculate current alarm cycle time
		
		alarm_cycle = alarm_cycle * 2;
			
		if (alarm_cycle > ALARM_CYCLE_MAX_SEC)
		{
			alarm_cycle = ALARM_CYCLE_MAX_SEC;
		}
		
		// count down alarm cycle
		tick = alarm_cycle;
		while ((SWITCH) && (tick--))
		{
			led_blink(1,10);
			sleep(990);
		}
	}
}

int main(void)
{
	io_init();

	timers_init();
	
	// wait 3 s to stabilize timer2 clock
	LED_ON
	BUZZER_ON
	sleep(1000);
	
	LED_OFF
	BUZZER_OFF
	sleep(2000);
	
	while(1)
	{
		go_to_sleep();

		// check how long the switch if open and activate buzzer

		if (switch_active_long())
		{
			alarm();
		}
		
		// switch is closed now, go to sleep again in next loop instruction
	}
}
